"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetSocketClient = void 0;
const net_1 = require("net");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const common_1 = require("../../common");
const socket_1 = require("../../socket");
class NetSocketClient {
    constructor(options, eventStore, binaryMessageStore) {
        this._info = {
            type: options.type,
            address: options.address,
            originalAddress: options.address,
            connected: false,
            connecting: false,
        };
        this._options = options;
        this._eventStore = eventStore;
        this._binaryMessageStore = binaryMessageStore;
    }
    get info() {
        return { ...this._info };
    }
    get store() {
        return this._eventStore;
    }
    async connect() {
        if (!this._info.connected) {
            this.close();
            this._client = (0, net_1.createConnection)(this.netOptions());
            this._client.on('data', data => this._binaryMessageStore.add(data));
            this._binaryMessageStore.group().subscribe(message => {
                const unpacked = common_1.MessageUtil.unpack(message);
                const encoding = this._options.binaryEncoding;
                const eventMessage = {
                    format: unpacked.format,
                    size: unpacked.data.length,
                    data: unpacked.data,
                };
                if (encoding) {
                    eventMessage[encoding] = (Buffer.from(unpacked.data)).toString(encoding);
                }
                this._eventStore.add({
                    type: socket_1.SocketEventType.ReceivedMessage,
                    date: new Date(),
                    message: eventMessage,
                });
            });
            this._client.on('error', err => this._eventStore.add({
                type: socket_1.SocketEventType.Error,
                date: new Date(),
                message: {
                    message: err.message
                },
            }));
            this._client.on('close', () => {
                if (this._info.connected) {
                    this._eventStore.add({
                        type: socket_1.SocketEventType.Closed,
                        date: new Date(),
                        message: { code: 1, reason: '' },
                    });
                    this.close();
                    this.retryConnection();
                }
            });
            try {
                const connect$ = (0, rxjs_1.fromEvent)(this._client, 'connect').pipe(common_1.RxJSUtil.timeout(this._options.connectionTimeout, `connection to ${this.info.address} is timed out`), (0, operators_1.tap)(() => this._info.connected = true), (0, operators_1.tap)(() => this._eventStore.add({
                    type: socket_1.SocketEventType.Connected,
                    date: new Date(),
                    message: this._info,
                })), (0, operators_1.mapTo)(undefined));
                this._info.connecting = true;
                await (0, rxjs_1.firstValueFrom)(connect$);
                this._info.connecting = false;
                this._info.connected = true;
            }
            catch (error) {
                this.close();
                throw error;
            }
        }
    }
    async sendData(data) {
        const encoding = this._options.binaryEncoding;
        const packedString = common_1.MessageUtil.packToString(data);
        const packed = common_1.MessageUtil.packToBuffer(packedString.data);
        const eventMessage = {
            format: packed.format,
            size: packed.data.length,
            data: Array.from(packed.data),
        };
        if (encoding) {
            eventMessage[encoding] = packed.data.toString(encoding);
        }
        return this.send(Buffer.from(packed.data), eventMessage);
    }
    async sendBytes(bytes) {
        const encoding = this._options.binaryEncoding;
        const packed = common_1.MessageUtil.packToBuffer(bytes);
        const eventMessage = {
            format: packed.format,
            size: bytes.length,
            data: bytes
        };
        if (encoding) {
            eventMessage[encoding] = packed.data.toString(encoding);
        }
        return this.send(packed.data, eventMessage);
    }
    send(data, eventMessage) {
        const client = this._client;
        const connected = this._info.connected;
        if (!client || !connected) {
            throw new Error(`error: client is not connected to ${this.info.address}`);
        }
        return new Promise(async (resolve, reject) => {
            client.write(data, err => {
                if (!err) {
                    const event = {
                        type: socket_1.SocketEventType.SentMessage,
                        date: new Date(),
                        message: eventMessage,
                    };
                    const messageId = this._eventStore.add(event);
                    resolve({ messageId });
                }
                reject(err);
            });
        });
    }
    close() {
        this._client?.removeAllListeners();
        this._client = undefined;
        this._info.connected = false;
        this._info.connecting = false;
    }
    retryConnection() {
        const retryInterval = setInterval(async () => {
            try {
                if (this._info.connected) {
                    clearInterval(retryInterval);
                }
                else if (!this._info.connecting) {
                    this._eventStore.add({
                        type: socket_1.SocketEventType.Reconnecting,
                        date: new Date(),
                        message: this._info,
                    });
                    await this.connect();
                    clearInterval(retryInterval);
                }
            }
            catch (err) { }
        }, this._options.reconnectionInterval);
    }
    netOptions() {
        let netOpts = {};
        if (this._options.type === socket_1.SocketType.Tcp) {
            const parsedAddress = this._options.address.split(':');
            netOpts = {
                host: parsedAddress[0],
                port: parseInt(parsedAddress[1]),
            };
        }
        if (this._options.type === socket_1.SocketType.Unix) {
            netOpts = {
                path: this._options.address
            };
        }
        return netOpts;
    }
}
exports.NetSocketClient = NetSocketClient;
