"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetSocketSyncClient = void 0;
const net_1 = require("net");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const common_1 = require("../../common");
const socket_1 = require("../../socket");
class NetSocketSyncClient {
    constructor(options, eventStore) {
        this._options = options;
        this._eventStore = eventStore;
    }
    get store() {
        return this._eventStore;
    }
    async connect() {
        return undefined;
    }
    async sendData(data) {
        const encoding = this._options.binaryEncoding;
        const packedString = common_1.MessageUtil.packToString(data);
        const packed = common_1.MessageUtil.packToBuffer(packedString.data);
        const eventMessage = {
            format: packed.format,
            size: packed.data.length,
            data: Array.from(packed.data),
        };
        if (encoding) {
            eventMessage[encoding] = packed.data.toString(encoding);
        }
        return this.send(Buffer.from(packed.data), eventMessage);
    }
    async sendBytes(bytes) {
        const encoding = this._options.binaryEncoding;
        const packed = common_1.MessageUtil.packToBuffer(bytes);
        const eventMessage = {
            format: packed.format,
            size: bytes.length,
            data: bytes
        };
        if (encoding) {
            eventMessage[encoding] = packed.data.toString(encoding);
        }
        return this.send(packed.data, eventMessage);
    }
    async send(data, eventMessage) {
        const connectInfo = await this.connectSocket();
        const socketSendReplyPromise = new Promise((resolve, reject) => {
            connectInfo.socket.write(data, err => {
                if (!err) {
                    resolve({
                        messageId: this._eventStore.add({
                            type: socket_1.SocketEventType.SentMessage,
                            date: new Date(),
                            message: eventMessage,
                        })
                    });
                }
                reject(err);
            });
        });
        return (0, rxjs_1.firstValueFrom)((0, rxjs_1.from)(socketSendReplyPromise).pipe((0, rxjs_1.switchMap)(reply => (0, rxjs_1.merge)((0, rxjs_1.timer)(this._options.replyTimeout), (0, rxjs_1.fromEvent)(connectInfo.socket, 'close'), (0, rxjs_1.fromEvent)(connectInfo.socket, 'end')).pipe((0, rxjs_1.first)(), (0, operators_1.tap)(() => connectInfo.socket.destroy()), (0, operators_1.map)(() => connectInfo.buffer.length > 0
            ? { messageId: reply.messageId, reply: this.parseBuffer(connectInfo.buffer) }
            : reply)))));
    }
    async connectSocket() {
        const result = {
            socket: (0, net_1.createConnection)(this.netOptions()),
            buffer: Buffer.alloc(0),
        };
        result.socket.on('error', err => this._eventStore.add({
            type: socket_1.SocketEventType.Error,
            date: new Date(),
            message: {
                message: err.message
            },
        }));
        result.socket.on('close', () => {
            this._eventStore.add({
                type: socket_1.SocketEventType.Closed,
                date: new Date(),
                message: { code: 1, reason: '' },
            });
        });
        result.socket.on('data', chunk => {
            result.buffer = Buffer.concat([result.buffer, chunk]);
        });
        try {
            const connect$ = (0, rxjs_1.fromEvent)(result.socket, 'connect').pipe(common_1.RxJSUtil.timeout(1000, `connection to ${this._options.address} is timed out`), (0, operators_1.tap)(() => this._eventStore.add({
                type: socket_1.SocketEventType.Connected,
                date: new Date(),
                message: { type: this._options.type, address: this._options.address },
            })), (0, operators_1.mapTo)(undefined));
            await (0, rxjs_1.firstValueFrom)(connect$);
            return result;
        }
        catch (error) {
            result.socket.destroy();
            throw error;
        }
    }
    netOptions() {
        let netOpts = {};
        if (this._options.type === socket_1.SocketType.Tcp) {
            const parsedAddress = this._options.address.split(':');
            netOpts = {
                host: parsedAddress[0],
                port: parseInt(parsedAddress[1]),
            };
        }
        if (this._options.type === socket_1.SocketType.Unix) {
            netOpts = {
                path: this._options.address
            };
        }
        return netOpts;
    }
    parseBuffer(buffer) {
        const unpacked = common_1.MessageUtil.unpack(buffer);
        const encoding = this._options.binaryEncoding;
        const eventMessage = {
            format: unpacked.format,
            size: unpacked.data.length,
            data: unpacked.data,
        };
        if (encoding) {
            eventMessage[encoding] = (Buffer.from(unpacked.data)).toString(encoding);
        }
        this._eventStore.add({
            type: socket_1.SocketEventType.ReceivedMessage,
            date: new Date(),
            message: eventMessage,
        });
        return eventMessage;
    }
}
exports.NetSocketSyncClient = NetSocketSyncClient;
